/* 
 * @file
 * @author Amr Hassan <amr.hassan@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#ifndef CONNECTION_HPP_
#define CONNECTION_HPP_

#include <string>
#include <boost/shared_ptr.hpp>
#include "Exception.hpp"

typedef struct sqlite3 sqlite3;

namespace sqlite3pp
{
class Statement;

/**
 * A database connection
 */
class Connection
{
public:

	/**
	 * Opens a connection the database located at filePath
	 *
	 * @param uri The path of the local database file in UTF-8
	 *
	 * @throws Exception
	 */
	explicit Connection(const std::string& uri);

	/**
	 * Creates a SQL statement object.
	 *
	 * @param sql A UTF-8 SQL statement. You can substitute for values by the
	 * value ? and set it later using the functions
	 * Statement::setT(int number, T value), by doing so, all strings will be
	 * properly escaped first.
	 *
	 * @throws Exception
	 */
	Statement createStatement(const std::string& sql);

	/**
	 * A shortcut for creating a non-query statement and executing it.
	 * It can execute a set of SQL statements delimited by semi-colons.
	 *
	 * @param sql A UTF-8 SQL statement
	 *
	 * @throws Exception
	 */
	void execute(const std::string& sql);

private:
	friend class Statement;
	friend class ResultSet;

	boost::shared_ptr<sqlite3> internalObject;
};
}

#endif /* CONNECTION_HPP_ */
