/* 
 * @file
 * @author Amr Hassan <amr.hassan@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */


#include <sqlite3.h>
#include <string>
#include <boost/thread.hpp>
#include "Connection.hpp"
#include "ResultSet.hpp"
#include "Exception.hpp"
#include "Statement.hpp"
#include "rogue_private.hpp"

using namespace sqlite3pp;

bool sqlite3pp::ResultSet::next()
{
	boost::mutex::scoped_lock(mutex);

	int resultCode = sqlite3_step(internalStatement.get());
	checkResultCode(resultCode, internalConnection.get());

	if (resultCode == SQLITE_ROW) {
		return true;
	} else {
		if (resultCode == SQLITE_DONE) {
			_isDone = true;
		}
		return false;
	}
}

int sqlite3pp::ResultSet::readInteger(int column)
{
	checkColumnRange(column);
	return sqlite3_column_int(internalStatement.get(), column);
}

int sqlite3pp::ResultSet::getColumnCount()
{
	return sqlite3_column_count(internalStatement.get());
}

std::string sqlite3pp::ResultSet::readString(int column)
{
	checkColumnRange(column);
	return std::string(reinterpret_cast<const char*>(sqlite3_column_text(internalStatement.get(), column)));
}

void ResultSet::checkColumnRange(int column)
{
	if (column < 0 || column >= getColumnCount()) {
		throw Exception("Column out of range (it starts with 0, not 1)");
	}
}

bool ResultSet::isDone()
{
	return _isDone;
}

ResultSet::ResultSet(Statement& statementObject) : internalStatement(statementObject.internalStatement),
		internalConnection(statementObject.internalConnection)
{

}
