/* 
 * @file
 * @author Amr Hassan <amr.hassan@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#ifndef RESULTSET_H_
#define RESULTSET_H_

#include <string>
#include <boost/shared_ptr.hpp>
#include <boost/thread/recursive_mutex.hpp>
#include "Exception.hpp"

typedef struct sqlite3_stmt sqlite3_stmt;

namespace sqlite3pp
{
class Statement;

/**
 * A query result from an SQL statement
 */
class ResultSet
{
public:
	/**
	 * Move to the next row (record)
	 *
	 * @return true if moved on to a valid record. Will return false when reaches
	 * 			end of results
	 *
	 * @throws Exception
	 */
	bool next();

	/**
	 * Read the value at the given column number as a UTF-8 character string and
	 * return it
	 *
	 * @param column The column number in this record where the value is
	 *
	 * @throws Exception
	 */
	std::string readString(int column);

	/**
	 * Read the value at the given column number as an integer and return it
	 *
	 * @param column The column number in this record where the value is
	 *
	 * @throws Exception
	 */
	int readInteger(int column);

	/**
	 * Read the value at the given column number as a double and return it
	 *
	 * @param column The column number in this record where the value is
	 *
	 * @throws Exception
	 */
	double readDouble(int column);

	/**
	 * Check whether going over all the records is done
	 */
	bool isDone();

	/**
	 * Returns the number of columns
	 */
	int getColumnCount();

private:
	friend class Statement;

	boost::shared_ptr<sqlite3_stmt> internalStatement;
	boost::shared_ptr<sqlite3> internalConnection;
	bool _isDone;
	static boost::recursive_mutex mutex;		//TODO: Make this non-static

	ResultSet(Statement& statementObject);

	void checkColumnRange(int column);
};
}



#endif /* RESULTSET_H_ */
