/* 
 * @file
 * @author Amr Hassan <amr.hassan@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include <sqlite3.h>
#include <sstream>
#include <boost/thread.hpp>
#include "Statement.hpp"
#include "Connection.hpp"
#include "ResultSet.hpp"
#include "rogue_private.hpp"
#include "rogue.hpp"

using namespace sqlite3pp;

class StatementDeleter
{
public:
	void operator()(sqlite3_stmt* statement)
	{
		sqlite3_finalize(statement);
	}
};

void Statement::prepare()
{
	boost::mutex::scoped_lock(mutex);

	sqlite3_stmt* temp;
	const char* unused;
	int resultCode = sqlite3_prepare_v2(internalConnection.get(), unusedSql.c_str(), -1, &temp, &unused);
	unusedSql = unused;

	checkResultCode(resultCode, internalConnection.get());

	internalStatement = boost::shared_ptr<sqlite3_stmt>(temp, StatementDeleter());
}

sqlite3pp::Statement::Statement(const Connection& connection, const std::string& sql) :
		internalConnection(connection.internalObject), unusedSql(sql)
{
	bool opened = false;
	for (unsigned int i = 0; i < unusedSql.size(); i++) {
		if (unusedSql[i] == '\'') {
			opened = !opened;
		} else if (unusedSql[i] == '?' && !opened) {
			parameterLocations.push_back(i);
		}
	}
}

void Statement::execute()
{
	boost::mutex::scoped_lock(mutex);

	while (!unusedSql.empty()) {
		prepare();
		int resultCode = sqlite3_step(internalStatement.get());
		checkResultCode(resultCode, internalConnection.get());
	}
}

ResultSet Statement::executeQuery()
{
	boost::mutex::scoped_lock(mutex);

	prepare();
	return ResultSet(*this);
}

void Statement::setString(unsigned int arg, const std::string& value)
{
	boost::mutex::scoped_lock(mutex);

	if (arg > parameterLocations.size()) {
		throw Exception("Argument number out of bounds. Hint: They start from 0.");
	}

	std::string replacer = '\'' + escape(value) + '\'';

	unsigned int location = parameterLocations[arg];
	unusedSql.erase(location, 1);
	unusedSql.insert(location, replacer);

	for (unsigned int i = arg+1; i < parameterLocations.size(); i++) {
		parameterLocations[i] += replacer.size() - 1;
	}
}

void Statement::setInteger(unsigned int arg, long value)
{
	boost::mutex::scoped_lock(mutex);

	if (arg > parameterLocations.size()) {
		throw Exception("Argument number out of bounds. Hint: They start from 0.");
	}

	std::stringstream ss;
	ss << value;

	std::string replacer = ss.str();

	unsigned int location = parameterLocations[arg];
	unusedSql.erase(location, 1);
	unusedSql.insert(location, replacer);

	for (unsigned int i = arg+1; i < parameterLocations.size(); i++) {
		parameterLocations[i] += replacer.size() - 1;
	}
}

void Statement::setDouble(unsigned int arg, double value)
{
	boost::mutex::scoped_lock(mutex);

	if (arg > parameterLocations.size()) {
		throw Exception("Argument number out of bounds. Hint: They start from 0.");
	}

	std::stringstream ss;
	ss << value;

	std::string replacer = ss.str();

	unsigned int location = parameterLocations[arg];
	unusedSql.erase(location, 1);
	unusedSql.insert(location, replacer);

	for (unsigned int i = arg+1; i < parameterLocations.size(); i++) {
		parameterLocations[i] += replacer.size() - 1;
	}
}

std::size_t Statement::executeUpdate()
{
	execute();
	return sqlite3_changes(internalConnection.get());
}
