/* 
 * @file
 * @author Amr Hassan <amr.hassan@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#ifndef STATEMENT_H_
#define STATEMENT_H_

#include <string>
#include <vector>
#include <boost/shared_ptr.hpp>
#include <boost/thread/recursive_mutex.hpp>
#include "Exception.hpp"

typedef struct sqlite3_stmt sqlite3_stmt;
typedef struct sqlite3 sqlite3;

namespace sqlite3pp
{
class Connection;
class ResultSet;

/**
 * An SQL statement (actually a prepared statement)
 */
class Statement
{
public:

	/**
	 * Creates an SQL Statement object
	 *
	 * @param sql A UTF-8 SQL statement. You can substitute for values by the
	 * value ? and set it later using the functions setT(int number, T value),
	 * by doing so, all strings will be properly escaped first.
	 */
	Statement(const Connection& connection, const std::string& sql);

	/**
	 * Execute this non-query statement. It can execute a set of semi-colon statements.
	 *
	 * @throws Exception
	 */
	void execute();

	/**
	 * Execute this query statement and return a ResultSet
	 *
	 * @throws Exception
	 */
	ResultSet executeQuery();

	/**
	 * Executes a INSERT, UPDATE or DELETE statement and returns the number of
	 * affected rows.
	 *
	 * @throws Exception
	 */
	std::size_t executeUpdate();

	/**
	 * Set parameter marked by placeholder ? to be a string value. It will
	 * be properly escaped first.
	 *
	 * @param arg The index of the placeholder ? in the statement
	 *
	 * @throws Exception
	 */
	void setString(unsigned int arg, const std::string& value);

	/**
	 * Set parameter marked by placeholder ? to be an integer value.
	 *
	 * @param arg The index of the placeholder ? in the statement
	 *
	 * @throws Exception
	 */
	void setInteger(unsigned int arg, long value);

	/**
	 * Set the parameter marked by placeholder ? to be an double value.
	 *
	 * @param arg The index of the placeholder ? in the statement
	 */
	void setDouble(unsigned int arg, double value);

private:
	friend class Connection;
	friend class ResultSet;

	void prepare();

	boost::shared_ptr<sqlite3_stmt> internalStatement;
	boost::shared_ptr<sqlite3> internalConnection;
	std::string unusedSql;
	std::vector<int> parameterLocations;
	static boost::recursive_mutex mutex;			// TODO: Make this non-static
};
}

#endif /* STATEMENT_H_ */
